@@ -1,4 +1,6 @@
 use std::collections::BTreeMap;
+use std::fmt::Display;
+use std::str::FromStr;
 
 use crate::{
     InputType, InputValueError, InputValueResult, Name, OutputType, Scalar, ScalarType, Value,
@@ -6,15 +8,24 @@ use crate::{
 
 /// A scalar that can represent any JSON Object value.
 #[Scalar(internal, name = "JSONObject")]
-impl<T> ScalarType for BTreeMap<String, T>
+impl<K, T> ScalarType for BTreeMap<K, T>
 where
+    K: ToString + FromStr + Ord + Sync + Send,
+    K::Err: Display,
     T: OutputType + InputType,
 {
     fn parse(value: Value) -> InputValueResult<Self> {
         match value {
             Value::Object(map) => map
                 .into_iter()
-                .map(|(name, value)| Ok((name.to_string(), T::parse(Some(value))?)))
+                .map(|(name, value)| {
+                    Ok((
+                        K::from_str(&name).map_err(|err| {
+                            InputValueError::custom(format!("object key: {}", err))
+                        })?,
+                        T::parse(Some(value))?,
+                    ))
+                })
                 .collect::<Result<_, _>>()
                 .map_err(InputValueError::propagate),
             _ => Err(InputValueError::expected_type(value)),
@@ -24,7 +35,7 @@ where
     fn to_value(&self) -> Value {
         let mut map = BTreeMap::new();
         for (name, value) in self {
-            map.insert(Name::new(name), value.to_value());
+            map.insert(Name::new(name.to_string()), value.to_value());
         }
         Value::Object(map)
     }
